Package: stringdist
Maintainer: Mark van der Loo <mark.vanderloo@gmail.com>
License: GPL-3
Title: Approximate String Matching, Fuzzy Text Search, and String
        Distance Functions
Type: Package
LazyLoad: yes
Authors@R: c( person("Mark", "van der Loo", role=c("aut","cre")
             , email="mark.vanderloo@gmail.com"
             , comment= c(ORCID="0000-0002-9807-4686"))
            , person("Jan", "van der Laan", role="ctb")
            , person("R Core Team",""     , role="ctb")
            , person("Nick","Logan"       , role="ctb")
            , person("Chris","Muir"       , role="ctb")
            , person("Johannes", "Gruber" , role="ctb")
            , person("Brian","Ripley"     , role="ctb"))
Description: Implements an approximate string matching version of R's native
    'match' function. Also offers fuzzy text search based on various string
     distance measures. Can calculate various string distances based on edits
    (Damerau-Levenshtein, Hamming, Levenshtein, optimal sting alignment), qgrams (q-
    gram, cosine, jaccard distance) or heuristic metrics (Jaro, Jaro-Winkler). An
    implementation of soundex is provided as well. Distances can be computed between
    character vectors while taking proper care of encoding or between integer
    vectors representing generic sequences. This package is built for speed and
    runs in parallel by using 'openMP'. An API for C or C++ is exposed as well.
    Reference: MPJ van der Loo (2014) <doi:10.32614/RJ-2014-011>.
Version: 0.9.17
Depends: R (>= 2.15.3)
URL: https://github.com/markvanderloo/stringdist
BugReports: https://github.com/markvanderloo/stringdist/issues
Suggests: tinytest
Imports: parallel
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-01-16 15:16:37 UTC; mark
Author: Mark van der Loo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9807-4686>),
  Jan van der Laan [ctb],
  R Core Team [ctb],
  Nick Logan [ctb],
  Chris Muir [ctb],
  Johannes Gruber [ctb],
  Brian Ripley [ctb]
Repository: CRAN
Date/Publication: 2026-01-16 17:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 09:11:59 UTC; unix
Archs: stringdist.so.dSYM
