# Generated by roxygen2: do not edit by hand

export(afind)
export(ain)
export(amatch)
export(extract)
export(grab)
export(grabl)
export(phonetic)
export(printable_ascii)
export(qgrams)
export(seq_ain)
export(seq_amatch)
export(seq_dist)
export(seq_distmatrix)
export(seq_qgrams)
export(seq_sim)
export(stringdist)
export(stringdistmatrix)
export(stringsim)
export(stringsimmatrix)
importFrom(parallel,detectCores)
useDynLib(stringdist, .registration=TRUE)
